from flask import Flask, request
from flask import jsonify
from doi import doi_parser
from zbl import zbl_parser
from mr import mr_parser
from all import all_parser
from shared import json_parser
from shared import xml_parser
from shared import txt_parser
from mr import mr_advanced_parser
from mr import mr_advanced_finder
from config.swagger import swagger_config 
from flasgger import Swagger, swag_from
from datetime import timedelta
import requests_cache

#Set up of Flask REST API
app = Flask(__name__)
#Set up for Swagger UI generated by flasgger
app.config["SWAGGER"] = {"title": "Swagger-UI", "uiversion": 3,'openapi': '3.0.3'}
swagger = Swagger(app, config=swagger_config)

#Installig global requests cache to cache all requests in all sesssions, which is going to expire after 14 days
requests_cache.install_cache(
    cache_name='requests_cache', expire_after=timedelta(days=14))

def response_ok_with_jsonify(list_output):
    """This function creates a successful response with text and status code 200

    Args:
        list_output (List[Dict[str]]): List of dictionaries, which is going to be serialized as an response

    Returns:
        Any: Flask response with successful status code 200  
    """
    response = jsonify(list_output)
    response.status_code = 200
    return response


def response_unsupported_format():
    """This function creates an unsuccessful response with status code 415, because selected Content-Type is not supported

    Returns:
        Any: Flask response with unsuccessful status code 415 
    """
    response = jsonify([])
    response.status_code = 415
    return response

def response_semantic_error():
    """This function creates an unsuccessful response with status code 422, because request body was semantically incorrect, see sample_data

    Returns:
        Any: Flask response with unsuccessful status code 422
    """
    response = jsonify([])
    response.status_code = 422
    return response

def response_service_unavailable():
    """This function creates an unsuccessful response with status code 503, because server can't reach connection to API to retrieve identifiers

    Returns:
        Any: Flask response with unsuccessful status code 503
    """
    response = jsonify()
    response.status_code = 503
    return response

def parse_request(parser):
    """This function parse request due to Content-Type with selected parser for DOI, ZBL, MR or all of them

    Args:
        parser (Callable): Function, which is going to call parser for DOI, ZBL, MR or all of them 

    Returns:
        Any: Successful response if Content-Type and body of the request was correct, unsuccessful response otherwise
    """
    try:
        if request.content_type == "application/xml":
            return response_ok_with_jsonify(xml_parser.parse_xml(request.data, parser))
        if request.content_type == "text/plain":
            return response_ok_with_jsonify(txt_parser.parse_txt(request.data, parser))
        if request.content_type == "application/json":
            return response_ok_with_jsonify(json_parser.parse_json(request.data, parser))
        return response_unsupported_format()
    except ConnectionError:
        return response_service_unavailable()
    except:
        return response_semantic_error()


@app.route("/doi", methods=["POST"])
@swag_from("./docs/doi_config.yml")
def doi_parse():
    """This function is going to parse request from user, which was sent as a POST method to endpoint /doi

    Returns:
        Any: Response extended by retrieved DOI, DOI score and DOI link
    """
    return parse_request(doi_parser.parser)


@app.route("/zbl", methods=["POST"])
@swag_from("./docs/zbl_config.yml")
def zbl_parse():
    """This function is going to parse request from user, which was sent as a POST method to endpoint /zbl

    Returns:
        Any: Response extended by retrieved ZBL and its link
    """
    return parse_request(zbl_parser.parser)


@app.route("/mr", methods=["POST"])
@swag_from("./docs/mr_config.yml")
def mr_parse():
    """This function first of all checks if user has access to advanced search of identifier MR
       then it is going to parse request from user, which was sent as a POST method to endpoint /mr

    Returns:
        Any: Response extended by retrieved MR and its link
    """
    if mr_advanced_finder.check_access_to_advanced_search():
        return parse_request(mr_advanced_parser.parser)
    return parse_request(mr_parser.parser)


@app.route("/all", methods=["POST"])
@swag_from("./docs/all_config.yml")
def all_parse():
    """This function is going to parse request from user, which was sent as a POST method to endpoint /all

    Returns:
        Any: Response extended by retrieved DOI, DOI score, ZBL, MR and it links
    """
    return parse_request(all_parser.parser)


if __name__ == '__main__':
    app.run(host='0.0.0.0')